/******************************************************
*
*  Copyright:    Infinetix Incorporated, 2001
*
*  Program File: bq2040.cpp
*  Author:       Dan Parker
*
*  Description:  This file contains the Cbq2040 class
*		 member functions.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*******************************************************/

#include "bq2040.h"

#define FALSE              0
#define TRUE               1
#define BQ2040_ADDRESS     0x0B

typedef union {
   unsigned short w;
   short i;
   unsigned char b[2];
} WORD16;

Cbq2040::Cbq2040(void) : Ci2cDev(BQ2040_ADDRESS)
{
}

Cbq2040::~Cbq2040(void)
{
}

int Cbq2040::read( unsigned int code, unsigned char *data)
{
   int result = FALSE;
   int size;

   if( ((code >= 0x1D) && (code <= 0x1F)) ||
       ((code > 0x23) && ((code != 0x2F) && (code != 0x3E) && (code != 0x3F))) )
      size = 0;
   else
      switch (code)
      {
         case 0x20: size = 11;
                    break;
         case 0x21: size = 7;
                    break;
         case 0x22:
         case 0x23: size = 5;
                    break;
         default:   size = 2;
                    break;
      }

   if( size > 0 )
      result = I2cRead( code, size, data );

   return result;
}

int Cbq2040::write( unsigned char Code, unsigned short Value )
{
   int result = FALSE;
   WORD16 temp;
   unsigned char data[3];

   if( Code <= 0x04 )
   {
      data[0] = Code;
      temp.w = Value;
      data[1] = temp.b[0];
      data[2] = temp.b[1];
      result = I2cWrite( data, 3 );
   }
   return result;
}

int Cbq2040::ManufacturerAccess( unsigned short *ReadData )
{
   WORD16 temp;
   int result;

   result = read( 0x00, &temp.b[0] );
   *ReadData = temp.w;
   return result;
}

int Cbq2040::WriteManufacturerAccess( unsigned short Value )
{
   return write( 0x00, Value );
}

int Cbq2040::RemainingCapacityAlarm( unsigned short *ReadData )
{
   WORD16 temp;
   int result;

   result = read( 0x01, &temp.b[0] );
   *ReadData = temp.w;
   return result;
}

int Cbq2040::WriteRemainingCapacityAlarm( unsigned short Value )
{
   return write( 0x01, Value );
}

int Cbq2040::RemainingTimeAlarm( unsigned short *ReadData )
{
   WORD16 temp;
   int result;

   result = read( 0x02, &temp.b[0] );
   *ReadData = temp.w;
   return result;
}

int Cbq2040::WriteRemainingTimeAlarm( unsigned short Value )
{
   return write( 0x02, Value );
}

int Cbq2040::BatteryMode( unsigned short *ReadData )
{
   WORD16 temp;
   int result;

   result = read( 0x03, &temp.b[0] );
   *ReadData = temp.w;
   return result;
}

int Cbq2040::WriteBatteryMode( unsigned short Value )
{
   return write( 0x03, Value );
}

int Cbq2040::AtRate( short *ReadData )
{
   WORD16 temp;
   int result;

   result = read( 0x04, &temp.b[0] );
   *ReadData = temp.i;
   return result;
}

int Cbq2040::WriteAtRate( short Value )
{
   WORD16 temp;

   temp.i = Value;
   return write( 0x04, temp.w );
}

int Cbq2040::AtRateTimeToFull( unsigned short *ReadData )
{
   int result;
   WORD16 current, time;

   result = AverageCurrent( &current.i );
   if( (result == 2) && (current.i >= 0) )
   {
      result = WriteAtRate(current.i);
      if( result == 3 )
      {
         result = read( 0x05, &time.b[0] );
         if( result == 2 )
            *ReadData = time.i;
      }
   }
   else
      result = 0;

   return result;
}

int Cbq2040::AtRateTimeToEmpty( unsigned short *ReadData )
{
   int result;
   WORD16 current, time;

   result = AverageCurrent( &current.i );
   if( (result == 2) && (current.i <= 0) )
   {
      result = WriteAtRate(current.i);
      if( result == 3 )
      {
         result = read( 0x06, &time.b[0] );
         if( result != 2 )
            *ReadData = time.i;
      }
   }
   else
      result = 0;

   return result;
}

int Cbq2040::AtRateOK( unsigned short *ReadData )
{
   return read( 0x07, (unsigned char *)(ReadData) );
}

int Cbq2040::Temperature( unsigned short *ReadData )
{
   return read( 0x08, (unsigned char *)(ReadData) );
}

int Cbq2040::Voltage( unsigned short *ReadData )
{
   return read( 0x09, (unsigned char *)(ReadData) );
}

int Cbq2040::Current( short *ReadData )
{
   return read( 0x0A, (unsigned char *)(ReadData) );
}

int Cbq2040::AverageCurrent( short *ReadData )
{
   return read( 0x0B, (unsigned char *)(ReadData) );
}

int Cbq2040::MaxError( unsigned short *ReadData )
{
   return read( 0x0C, (unsigned char *)(ReadData) );
}

int Cbq2040::RelativeStateOfCharge( unsigned short *ReadData )
{
   unsigned short CapacityValue;
   int result;

   result = DesignCapacity( &CapacityValue );
   if( (result == 2) && 
       (CapacityValue > 1504) && 
       (CapacityValue < 10400 ) )
      result = read( 0x0D, (unsigned char *)(ReadData) );
   else
      result = 0;

   return result;
}

int Cbq2040::AbsoluteStateOfCharge( unsigned short *ReadData )
{
   unsigned short CapacityValue;
   int result;

   result = DesignCapacity( &CapacityValue );
   if( (result == 2) && 
       (CapacityValue > 1504) && 
       (CapacityValue < 10400 ) )
      result = read( 0x0E, (unsigned char *)(ReadData) );
   else
      result = 0;

   return result;
}

int Cbq2040::RemainingCapacity( unsigned short *ReadData )
{
   return read( 0x0F, (unsigned char *)(ReadData) );
}

int Cbq2040::FullChargeCapacity( unsigned short *ReadData )
{
   return read( 0x10, (unsigned char *)(ReadData) );
}

int Cbq2040::RunTimeToEmpty( unsigned short *ReadData )
{
   return read( 0x11, (unsigned char *)(ReadData) );
}

int Cbq2040::AverageTimeToEmpty( unsigned short *ReadData )
{
   return read( 0x12, (unsigned char *)(ReadData) );
}

int Cbq2040::AverageTimeToFull( unsigned short *ReadData )
{
   return read( 0x13, (unsigned char *)(ReadData) );
}

int Cbq2040::ChargingCurrent( unsigned short *ReadData )
{
   return read( 0x14, (unsigned char *)(ReadData) );
}

int Cbq2040::ChargingVoltage( unsigned short *ReadData )
{
   return read( 0x15, (unsigned char *)(ReadData) );
}

int Cbq2040::BatteryStatus( unsigned short *ReadData )
{
   return read( 0x16, (unsigned char *)(ReadData) );
}

int Cbq2040::CycleCount( unsigned short *ReadData )
{
   return read( 0x17, (unsigned char *)(ReadData) );
}

int Cbq2040::DesignCapacity( unsigned short *ReadData )
{
   return read( 0x18, (unsigned char *)(ReadData) );
}

int Cbq2040::DesignVoltage( unsigned short *ReadData )
{
   return read( 0x19, (unsigned char *)(ReadData) );
}

int Cbq2040::SpecificationInfo( unsigned short *ReadData )
{
   return read( 0x1A, (unsigned char *)(ReadData) );
}

int Cbq2040::ManufactureDate( unsigned short *ReadData )
{
   return read( 0x1B, (unsigned char *)(ReadData) );
}

int Cbq2040::SerialNumber( unsigned short *ReadData )
{
   return read( 0x1C, (unsigned char *)(ReadData) );
}

int Cbq2040::ManufacturerName( unsigned char *ReadData )
{
   return read( 0x20, ReadData );
}

int Cbq2040::DeviceName( unsigned char *ReadData )
{
   return read( 0x21, ReadData );
}

int Cbq2040::DeviceChemistry( unsigned char *ReadData )
{
   return read( 0x22, ReadData );
}

int Cbq2040::ManufacturerData( unsigned char *ReadData )
{
   return read( 0x23, ReadData );
}

int Cbq2040::Flags( unsigned short *ReadData )
{
   return read( 0x2F, (unsigned char *)(ReadData) );
}

int Cbq2040::EndOfDischargeVoltage1( unsigned short *ReadData )
{
   return read( 0x3E, (unsigned char *)(ReadData) );
}

int Cbq2040::EndOfDischargeVoltageF( unsigned short *ReadData )
{
   return read( 0x3F, (unsigned char *)(ReadData) );
}

