/******************************************************
*
*  Copyright:    Infinetix Incorporated, 2001
*
*  Program File: test7118.cpp
*  Author:       Dan Parker
*
*  Description:  This program tests the Csaa7118 class.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*******************************************************/

/* Standard C library includes                        */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* SAA7118 class include                              */
#include "saa7118.h"

#define FALSE       0
#define TRUE        1
#define DATA_SZ     2

void PrintData( int size, unsigned char *Data )
{
   int DataIndex;

   for( DataIndex = 0; DataIndex < size; ++DataIndex )
   {
      printf( "%X ", Data[DataIndex] );
      if( ((DataIndex + 1) % 0x10) == 0 )
         printf( "\n" );
   }
   printf( "\n" );

}

int atohex( char *cstring )
{
   int index = 0;
   int valout = 0;
   char c;

   while( cstring[index] != '\0' )
   {
      c = cstring[index];
      if( (c >= '0') && (c <= '9') )
         valout = (valout * 16) + (int)(c - '0');
      if( (c >= 'a') && (c <= 'f') )
         valout = (valout * 16) + (int)(c - 'a') + 10;
      if( (c >= 'A') && (c <= 'F') )
         valout = (valout * 16) + (int)(c - 'A') + 10;

      ++index;
   }

   return valout;
}

int main( int argc, char *argv[] )
{
   int code;
   Csaa7118 ViedoChip;
   unsigned char DataBuffer[DATA_SZ];
   int result, index;

   if( ( (argc == 2) && (strcmp("read", argv[1]) == 0) ) ||
       ( (argc == 4) && (strcmp("write", argv[1]) == 0 ) ) )
   {
      if( argc == 2 )
      {
         /* Print column header */
         printf( "   " );
         for( index = 0; index < 0x10; ++index )
            printf( "%3X", index );

         /* Print data */
         index = 0;
         for( code = 0; code <= 0xEF; ++code )
         {
            /* Print row header */
            if( (code % 0x10) == 0 )
            {
               printf( "\n" );
               printf( "%X0 ", index );
               ++index;
            }
            result = ViedoChip.read( code, 1, DataBuffer );
            if( result == 0 )
               printf( "   " );
            else
               printf( "%3X", DataBuffer[0] );
         }
         printf( "\n" );
      }
      else
      {
         DataBuffer[0] = atohex(argv[2]);
         DataBuffer[1] = atohex(argv[3]);
         result = ViedoChip.write( DataBuffer, DATA_SZ );
         printf( "test7118 write result: %i   ", result );
         printf( "retry count: %i\n", ViedoChip.RetryCount );
      }
   }
   else
   {
      printf("Usage: testvid read\n");
      printf("       testvid write <hex code> <hex value>\n");
   }

   return 0;
}
