/******************************************************
*
*  Copyright:    Infinetix Incorporated, 2001
*
*  Program File: testbq2040.cpp
*  Author:       Dan Parker
*
*  Description:  This program tests the Cbq2040 class.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*******************************************************/

/* Standard C library includes                        */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* BQ2040 class include                               */
#include "bq2040.h"

#define FALSE       0
#define TRUE        1
#define DATA_SZ     80

void PrintData( unsigned int Code, unsigned char *Data )
{
   int DataIndex;
   int ByteCount;

   switch (Code)
   {
      case 0x20: ByteCount = 11;
                 break;
      case 0x21: ByteCount = 7;
                 break;
      case 0x22:
      case 0x23: ByteCount = 5;
                 break;
      default:   ByteCount = 2;
                 break;
   }

   for( DataIndex = 0; DataIndex < ByteCount; ++DataIndex )
   {
      printf( "%X ", Data[DataIndex] );
      if( ((DataIndex + 1) % 0x10) == 0 )
         printf( "\n" );
   }
   printf( "\n" );

}

int atohex( char *cstring )
{
   int index = 0;
   int valout = 0;
   char c;

   while( cstring[index] != '\0' )
   {
      c = cstring[index];
      if( (c >= '0') && (c <= '9') )
         valout = (valout * 16) + (int)(c - '0');
      if( (c >= 'a') && (c <= 'f') )
         valout = (valout * 16) + (int)(c - 'a') + 10;
      if( (c >= 'A') && (c <= 'F') )
         valout = (valout * 16) + (int)(c - 'A') + 10;

      ++index;
   }

   return valout;
}

int main( int argc, char *argv[] )
{
   Cbq2040 ps;
   unsigned short USWord;
   short SWord;
   unsigned char DataBuffer[DATA_SZ];
   int result;

   if( !((argc == 2) ||
        ((argc == 4) && (strcmp("write", argv[1]) == 0))) )
   {
      printf("Usage: testps <hex code>{read} | 1<hex code>{access function}\n");
      printf("       testps write <hex code> <hex data word>\n");
      return 0;
   }

   if( strcmp("write", argv[1]) != 0 )
   {
      switch( atohex(argv[1]) )
      {
         case 0x100:
            result = ps.ManufacturerAccess(&USWord);
            if( result == 2 )
               printf( "ManufacturerAccess: 0x%X\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x101:
            result = ps.RemainingCapacityAlarm(&USWord);
            if( result == 2 )
               printf( "RemainingCapacityAlarm: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x102:
            result = ps.RemainingTimeAlarm(&USWord);
            if( result == 2 )
               printf( "RemainingTimeAlarm: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x103:
            result = ps.BatteryMode(&USWord);
            if( result == 2 )
               printf( "BatteryMode: 0x%X\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x104:
            result = ps.AtRate(&SWord);
            if( result == 2 )
               printf( "AtRate: %i\n", SWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x105:
            result = ps.AtRateTimeToFull(&USWord);
            if( result == 2 )
               printf( "AtRateTimeToFull: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x106:
            result = ps.AtRateTimeToEmpty(&USWord);
            if( result == 2 )
               printf( "AtRateTimeToEmpty: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x107:
            result = ps.AtRateOK(&USWord);
            if( result == 2 )
               printf( "AtRateOK: 0x%X\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x108:
            result = ps.Temperature(&USWord);
            if( result == 2 )
               printf( "Temperature: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x109:
            result = ps.Voltage(&USWord);
            if( result == 2 )
               printf( "Voltage: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x10A:
            result = ps.Current(&SWord);
            if( result == 2 )
               printf( "Current: %i\n", SWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x10B:
            result = ps.AverageCurrent(&SWord);
            if( result == 2 )
               printf( "AverageCurrent: %i\n", SWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x10C:
            result = ps.MaxError(&USWord);
            if( result == 2 )
               printf( "MaxError: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x10D:
            result = ps.RelativeStateOfCharge(&USWord);
            if( result == 2 )
               printf( "RelativeStateOfCharge: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x10E:
            result = ps.AbsoluteStateOfCharge(&USWord);
            if( result == 2 )
               printf( "AbsoluteStateOfCharge: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x10F:
            result = ps.RemainingCapacity(&USWord);
            if( result == 2 )
               printf( "RemainingCapacity: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x110:
            result = ps.FullChargeCapacity(&USWord);
            if( result == 2 )
               printf( "FullChargeCapacity: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x111:
            result = ps.RunTimeToEmpty(&USWord);
            if( result == 2 )
               printf( "RunTimeToEmpty: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x112:
            result = ps.AverageTimeToEmpty(&USWord);
            if( result == 2 )
               printf( "AverageTimeToEmpty: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x113:
            result = ps.AverageTimeToFull(&USWord);
            if( result == 2 )
               printf( "AverageTimeToFull: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x114:
            result = ps.ChargingCurrent(&USWord);
            if( result == 2 )
               printf( "ChargingCurrent: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x115:
            result = ps.ChargingVoltage(&USWord);
            if( result == 2 )
               printf( "ChargingVoltage: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x116:
            result = ps.BatteryStatus(&USWord);
            if( result == 2 )
               printf( "BatteryStatus: 0x%X\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x117:
            result = ps.CycleCount(&USWord);
            if( result == 2 )
               printf( "CycleCount: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x118:
            result = ps.DesignCapacity(&USWord);
            if( result == 2 )
               printf( "DesignCapacity: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x119:
            result = ps.DesignVoltage(&USWord);
            if( result == 2 )
               printf( "DesignVoltage: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x11A:
            result = ps.SpecificationInfo(&USWord);
            if( result == 2 )
               printf( "SpecificationInfo: 0x%X\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x11B:
            result = ps.ManufactureDate(&USWord);
            if( result == 2 )
               printf( "ManufactureDate: 0x%X\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x11C:
            result = ps.SerialNumber(&USWord);
            if( result == 2 )
               printf( "SerialNumber: 0x%X\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x120:
            result = ps.ManufacturerName(DataBuffer);
            if( result == 11 )
            {
               printf( " ManufacturerName:  " );
               PrintData( 0x20, DataBuffer );
            }
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x121:
            result = ps.DeviceName(DataBuffer);
            if( result == 7 )
            {
               printf( " DeviceName:  " );
               PrintData( 0x21, DataBuffer );
            }
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x122:
            result = ps.DeviceChemistry(DataBuffer);
            if( result == 5 )
            {
               printf( " DeviceChemistry:  " );
               PrintData( 0x22, DataBuffer );
            }
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x123:
            result = ps.ManufacturerData(DataBuffer);
            if( result == 5 )
            {
               printf( " ManufacturerData:  " );
               PrintData( 0x23, DataBuffer );
            }
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x12F:
            result = ps.Flags(&USWord);
            if( result == 2 )
               printf( "Flags: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x13E:
            result = ps.EndOfDischargeVoltage1(&USWord);
            if( result == 2 )
               printf( "EndOfDischargeVoltage1: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         case 0x13F:
            result = ps.EndOfDischargeVoltageF(&USWord);
            if( result == 2 )
               printf( "EndOfDischargeVoltageF: %u\n", USWord ); 
            else
               printf( "ERROR result: %i\n", result );
            break;
         default:
            result = ps.read(atohex(argv[1]), DataBuffer);
            printf( "testbq2040 read result: %i  ", result ); 
            if( result > 0  )
               PrintData( atohex(argv[1]), DataBuffer );
            break;
      }
   }
   else
   {
      switch( atohex(argv[2]) )
      {
         case 0x00:
            USWord = atohex(argv[3]);
            result = ps.WriteManufacturerAccess(USWord);
            break;
         case 0x01:
            USWord = atohex(argv[3]);
            result = ps.WriteRemainingCapacityAlarm(USWord);
            break;
         case 0x02:
            USWord = atohex(argv[3]);
            result = ps.WriteRemainingTimeAlarm(USWord);
            break;
         case 0x03:
            USWord = atohex(argv[3]);
            result = ps.WriteBatteryMode(USWord);
            break;
         case 0x04:
            SWord = atohex(argv[3]);
            result = ps.WriteAtRate(SWord);
            break;
         default: 
            result = 0;
            break;
      }
      printf( "testbq2040 write result: %i  ", result ); 
   }
   printf( "retry count: %i\n", ps.RetryCount ); 

   return 0;
}
